##############################################################################
#
# Copyright (c) 2002 Andy McKay
# All rights reserved.
#
# Distributed under the following license http://www.agmweb.ca/Scripts/license
#
##############################################################################

from Globals import DTMLFile, MessageDialog

from OFS.SimpleItem import SimpleItem
from OFS.PropertyManager import PropertyManager
from OFS.Cache import Cacheable
from OFS.History import Historical
from ZPublisher.Converters import get_converter
from webdav.Lockable import ResourceLockedError
from webdav.WriteLockInterface import WriteLockInterface

manage_addPropertyListForm = DTMLFile('dtml/PropertyListAdd', globals())

defaultData = """
# Property List lets you put a list of properties in this object
# Lines starting with a # are a comment
# Examples:
# foo:string=bar
# This is property foo, that is a string of value bar
# The possible conversion types can be found in the Zope documentation
"""

def manage_addPropertyList(self, id, title='', data='', REQUEST=None):
    ''' Adds a new object '''
    self._setObject(id, PropertyList(id, title, data))
    if REQUEST is not None:
            return self.manage_main(self, REQUEST)

class PropertyList(SimpleItem, PropertyManager, Cacheable, Historical):
    meta_type = 'PropertyList'

    manage_main = DTMLFile('dtml/PropertyListEdit', globals())

    manage_options = ({'label': 'Edit', 'action': 'manage_main'},) \
        + SimpleItem.manage_options \
        + Historical.manage_options \
        + Cacheable.manage_options

    __ac_permissions__=(
        ('View management screens', ('manage_edit','document_src')),
        ('View', ('', 'index_html',) ),
        )

    __implements__ = (WriteLockInterface,)

    def __init__(self, id, title ='', data=''):
        """initialise a new instance of PropertyList"""
        self.id = id
        self.title = title
        if not data: data = defaultData
        self._data = data
        self._parse
        self._properties=[]

    def manage_edit(self, data='', title='', REQUEST=None):
        ''' clean up '''
        if self.wl_isLocked():
            raise ResourceLockedError, "The script is locked via WebDAV."

        self.title = title
        self._data = data
        self._parse()
        if REQUEST is not None:
            self.REQUEST.RESPONSE.redirect(self.absolute_url(1)+'/manage_main')

    def _parse(self):
        # parse data
        # delete the old stuff out
        for property in self._properties: delattr(self, property['id'])

        d = self._data
        lines = d.split('\n')
        lineNum = 0
        map = []
        for line in lines:
            lineNum = lineNum + 1
            line = line.strip()
            if not line or line[0] == '#': continue

            try:
                # from FSPropertiesObject ripped...
                propname, proptv = line.split(':' )
                proptype, propvstr = proptv.split( '=', 1 )

                propname = propname.strip()
                proptype = proptype.strip()
                propvstr = propvstr.strip()

                converter = get_converter(proptype, lambda x: x)
                propvalue = converter(propvstr.strip())

            except:
                raise ValueError, ( 'Error processing line %s of %s:\n%s' % (lineNum,self.getId(),line) )

#         This is what I would like to have, but the CMF is full of 
#         stuff with multiple names, so this barfs on all CMF stuff.
#
#            if not hasattr(self, propname):
            setattr(self, propname, propvalue)
#            else:
#                raise ValueError, ( 'Property already exists %s of %s:\n%s' % (lineNum,self.getId(),line) )

            map.append({'id':propname,
                        'type':proptype,
                        'mode':'',
                        'default_value':propvalue,
                        })

        self._properties = tuple(map)
        print self._properties

    def __str__(self):
        """ returns the string """
        return self._data

    def __call__(self, *args):
        """ some doc string """
        return self

    def document_src(self, *args):
        """ returns the string """
        return str(self)

    def PUT(self, REQUEST, RESPONSE):
        """ Handle HTTP PUT requests """
        self.dav__init(REQUEST, RESPONSE)
        self.dav__simpleifhandler(REQUEST, RESPONSE, refresh=1)
        self._data = REQUEST.get('BODY', '')
        self._parse()
        RESPONSE.setStatus(204)
        return RESPONSE

    index_html = __call__